module State.RoomsState
  ( RoomsState,
    initRoomsState,
    HasRoomsState (..),
    roomStateDiffers,
    RoomsStateDiff,
    roomStateDiffInOpenRooms,
    updateRoomState,
    getRoomState,
  )
where

import ClassyPrelude
import State.GenericTVarState
import State.RoomDataState (MonadRoomDataStateRead (getRoomDataState))
import Types.RoomData (RoomsData, RoomsStateDiff, roomNotEmpty, sameName)

type RoomsState = GenericTVarState RoomsData

initRoomsState :: IO RoomsState
initRoomsState = newTVarIO []

class HasRoomsState a where
  getRoomsState :: a -> RoomsState

updateRoomState ::
  ( HasRoomsState env,
    MonadIO m,
    MonadReader env m,
    MonadRoomDataStateRead m
  ) =>
  RoomsData ->
  m RoomsStateDiff
updateRoomState newData = do
  state <- getRoomsState <$> ask
  current <- getRoomDataState
  if not $ eqIgnoreOrdering newData current
    then
      ( do
          liftIO $ putStrLn "Upating room state"
          diff <- updateGenericTVarStateWithQuery state roomStateDiff newData
          liftIO $ putStrLn "Done Upating room state"
          return diff
      )
    else return ([], [])

getRoomState ::
  ( HasRoomsState env,
    MonadIO m,
    MonadReader env m
  ) =>
  m RoomsData
getRoomState = do
  state <- getRoomsState <$> ask
  getGenericTVarState state

roomStateDiffers ::
  ( MonadRoomDataStateRead m
  ) =>
  RoomsData ->
  m Bool
roomStateDiffers newData = do
  not . eqIgnoreOrdering newData <$> getRoomDataState

roomStateDiffInOpenRooms ::
  ( MonadRoomDataStateRead m,
    MonadIO m
  ) =>
  RoomsData ->
  m (RoomsData, RoomsData)
roomStateDiffInOpenRooms newData = do
  current <- getRoomDataState

  liftIO $ putStrLn $ pack $ "Current rooms: " ++ show current
  liftIO $ putStrLn $ pack $ "New rooms: " ++ show newData
  let newRooms = filter roomNotEmpty $ filter (\newRoom -> isNothing $ find (sameName newRoom) (filter roomNotEmpty current)) newData
  let oldRooms = filter (\oldRoom -> isNothing $ find (sameName oldRoom) newData) current

  return (newRooms, oldRooms)

roomStateDiff :: RoomsData -> RoomsData -> RoomsStateDiff
roomStateDiff newData current = (newRooms, oldRooms)
  where
    newRooms = filter roomNotEmpty $ filter (\newRoom -> isNothing $ find (sameName newRoom) (filter roomNotEmpty current)) newData
    oldRooms = filter (\oldRoom -> isNothing $ find (sameName oldRoom) newData) current

eqIgnoreOrdering :: (Eq a) => [a] -> [a] -> Bool
eqIgnoreOrdering a b = length a == length b && all (`elem` b) a
