{-# LANGUAGE DerivingVia #-}

module Types.AppTypes (Env (..), App (..), HasConfig (getConfig), getConnectedClientState, AppProfile (Prod, Dev)) where

import ClassyPrelude
import State.ConnectedClientsState
import State.RoomsState
  ( HasRoomsState (getRoomsState),
    RoomsState,
  )
import Types.Config (ServerOptions)

data AppProfile = Prod | Dev

data Env = Env
  { connectedClientsState :: ConnectedClientsState,
    roomsState :: RoomsState,
    profile :: AppProfile,
    config :: ServerOptions
  }

instance HasConnectedClientState Env where
  getConnectedClientState = connectedClientsState

instance HasRoomsState Env where
  getRoomsState = roomsState

class HasConfig a where
  getConfig :: a -> ServerOptions

instance HasConfig Env where
  getConfig = config

newtype App env a = App {unApp :: env -> IO a}
  deriving
    ( Functor,
      Applicative,
      Monad,
      MonadReader env,
      MonadIO
    )
    via ReaderT env IO
