module Types.Config
  ( ServerOptions (..),
    serverOptionsParser,
  )
where

import ClassyPrelude
  ( Applicative ((<*>)),
    Int,
    Semigroup ((<>)),
    Show,
    String,
    (<$>),
  )
import Options.Applicative (Parser, auto, help, long, metavar, option, short, showDefault, strOption, value)

data ServerOptions = ServerOptions
  { port :: Int, -- Main server port
    websocketPort :: Int, -- WebSocket server port
    listenAddress :: String, -- Address to bind the server
    notifyExecutable :: String -- Path to the notify executable
  }
  deriving (Show)

serverOptionsParser :: Parser ServerOptions
serverOptionsParser =
  ServerOptions
    <$> option
      auto
      ( long "port"
          <> short 'p'
          <> metavar "PORT"
          <> help "Port number for the main server (default: 8081)"
          <> value 8081
          <> showDefault
      )
    <*> option
      auto
      ( long "websocketPort"
          <> short 'w'
          <> metavar "WS_PORT"
          <> help "Port number for the WebSocket server (default: 9160)"
          <> value 9160
          <> showDefault
      )
    <*> strOption
      ( long "listenAddress"
          <> short 'l'
          <> metavar "ADDRESS"
          <> help "IP address or hostname to bind the server (default: 127.0.0.1)"
          <> value "127.0.0.1"
          <> showDefault
      )
    <*> strOption
      ( long "notifyExecutable"
          <> short 'n'
          <> metavar "EXECUTABLE"
          <> help "Path to the notify executable (default: /usr/bin/notify)"
          <> value "/usr/bin/notify"
          <> showDefault
      )
