{-# LANGUAGE DeriveGeneric #-}

module Types.RoomData (RoomData, RoomsStateDiff, RoomsData, sameName, roomNotEmpty, prettyPrintOpenedRoom) where

import ClassyPrelude
import Data.Aeson (FromJSON, ToJSON)
import Types.Participant (Participant (Participant, displayName))

data RoomData = RoomData
  { roomName :: RoomName,
    participants :: [Participant]
  }
  deriving (Generic, Show, Eq)

sameName :: RoomData -> RoomData -> Bool
sameName RoomData {roomName = name1} RoomData {roomName = name2} = name1 == name2

roomNotEmpty :: RoomData -> Bool
roomNotEmpty RoomData {participants = participants} = not $ null participants

prettyPrintOpenedRoom :: RoomData -> (Text, Text)
prettyPrintOpenedRoom RoomData {roomName = roomName, participants = participants} =
  (roomName, fromMaybe "" (headMay (map (\Participant {displayName = displayName} -> displayName) participants)))

type RoomName = Text

type RoomsData = [RoomData]

type RoomsStateDiff = (RoomsData, RoomsData)

instance ToJSON RoomData

instance FromJSON RoomData
