module Types.WebEnv
  ( HasWebEnv (..),
    WebEnv (..),
  )
where

import ClassyPrelude
import Network.Wai (Request, Response, ResponseReceived)
import State.ConnectedClientsState (HasConnectedClientState (getConnectedClientState))
import State.RoomsState (HasRoomsState (getRoomsState))
import Types.AppTypes
  ( Env (..),
    HasConfig (getConfig),
  )

class HasWebEnv a where
  getRequest :: a -> Request
  getRespond :: a -> (Response -> IO ResponseReceived)

data WebEnv = WebEnv
  { appEnv :: Env,
    request :: Request,
    respond :: Response -> IO ResponseReceived
  }

instance HasConnectedClientState WebEnv where
  getConnectedClientState = connectedClientsState . appEnv

instance HasRoomsState WebEnv where
  getRoomsState = roomsState . appEnv

instance HasWebEnv WebEnv where
  getRequest = request
  getRespond = respond

instance HasConfig WebEnv where
  getConfig = getConfig . appEnv
