{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE MonoLocalBinds #-}

module WebSocket.Server
  ( runWebSocketServer,
    runWSApp,
  )
where

import ClassyPrelude
import Data.UUID.V4 (nextRandom)
import Network.WebSockets qualified as WS
import Text.Printf
import Types.AppTypes (Env, HasConfig (getConfig))
import Types.Config (ServerOptions (..))
import WebSocket.WSApp (WSApp (..), WSEnv (..), wsApp)

runWebSocketServer ::
  ( MonadIO m,
    Types.AppTypes.HasConfig Types.AppTypes.Env,
    MonadReader Types.AppTypes.Env m
  ) =>
  m ()
runWebSocketServer = do
  config' <- getConfig <$> ask
  let ServerOptions {websocketPort = wsPort, listenAddress = address} = config'
  putStrLn $ pack $ printf "WebSocket server up and running at ws://%s:%d/" address wsPort
  wsApp' <- runWSApp
  liftIO $ WS.runServer address wsPort wsApp'

runWSApp ::
  ( MonadIO m,
    MonadReader Types.AppTypes.Env m
  ) =>
  m WS.ServerApp
runWSApp = do
  env <- ask
  return
    ( \pending -> do
        conn <- WS.acceptRequest pending
        uuid <- nextRandom
        let wsEnv = WSEnv {appEnv = env, connection = conn, clientId = uuid}
        WS.withPingThread conn 30 (return ()) $ unWSApp wsApp wsEnv
    )
