import { atom, useAtom } from 'jotai'
import { useCallback, useEffect } from 'react'
import { WEBSOCKET_URL } from '../background/constants'
import { UserInfo } from '../components/jitsi/types'


const webSocket = new WebSocket(WEBSOCKET_URL)
const webSocketConnectionAtom = atom(webSocket)

function useWebSocketConnection(userInfo: UserInfo) {
  console.log('[Rooms] useWebSocketConnection')

  const [webSocketConnection] = useAtom(webSocketConnectionAtom)
  useEffect(() => {
    sendMessageNowOrLater(webSocketConnection, JSON.stringify(userInfo))
  }, [webSocketConnection, userInfo]);

  const sendMessage = useCallback(
    (message: string) => {
      //if the socket's open, send a message:
      if (webSocketConnection.readyState === WebSocket.OPEN) {
        console.log('[Rooms] sending to ws', message)
        webSocketConnection.send(message)
      }
    },
    [webSocketConnection]
  )

  const onMessage = useCallback(
    (messageHandler: (messageHandler: string) => void) => {
      const wsMessageHandler = (ev: MessageEvent<any>) => {
        messageHandler(ev.data)
      }
      webSocketConnection.addEventListener('message', wsMessageHandler)
    },
    [webSocketConnection]
  )

  const disconnect = useCallback(webSocketConnection.close, [webSocketConnection])
  const cleanUpOnMessage = useCallback((callbackToRemove: any) => {
    console.log('[Rooms] cleanUpOnMessage')
    webSocketConnection.removeEventListener('message', callbackToRemove)
  }, [webSocketConnection])

  return { onMessage, cleanUpOnMessage, sendMessage, disconnect }
}

const sendMessageNowOrLater = (webSocket: WebSocket, message: string) => {
  if (webSocket.readyState !== WebSocket.OPEN) {
    return webSocket.addEventListener('open', (_: Event) => webSocket.send(message))
  }
  webSocket.send(message)
}

export default useWebSocketConnection
